@@ -9,6 +9,7 @@ use base 'Exporter';
 our @EXPORT = qw(blessed refaddr weaken longmess);
 
 use Carp ();
+use Carp::Heavy ();
 use Scalar::Util qw(blessed refaddr);
 
 # fake weaken if it isn't available.  will cause leaks, but this
@@ -17,7 +18,7 @@ use Scalar::Util qw(blessed refaddr);
   ? \&Scalar::Util::weaken
   : sub ($) { 0 };
 
-*longmess = $Carp::VERSION ? \&Carp::longmess : eval q{
+*longmess = !$Carp::VERSION ? eval q{
   package
     Carp;
   our (%CarpInternal, %Internal, $CarpLevel);
@@ -36,7 +37,15 @@ use Scalar::Util qw(blessed refaddr);
     local $CarpLevel = $CarpLevel + $level;
     &longmess;
   };
-};
+} : $Carp::VERSION <= 1.04 ? eval q{
+  package
+    Carp;
+  our ($CarpLevel);
+  sub {
+    local $INC{'Carp/Heavy.pm'} = $INC{'Carp/Heavy.pm'} || 1;
+    &longmess;
+  };
+} : \&Carp::longmess;
 
 if (defined &Carp::format_arg && $Carp::VERSION < 1.32) {
   my $format_arg = \&Carp::format_arg;
